@@ -1,27 +1,28 @@
-// <copyright file="DriverOptions.cs" company="WebDriver Committers">
+// <copyright file="DriverOptions.cs" company="Selenium Committers">
 // Licensed to the Software Freedom Conservancy (SFC) under one
-// or more contributor license agreements. See the NOTICE file
+// or more contributor license agreements.  See the NOTICE file
 // distributed with this work for additional information
-// regarding copyright ownership. The SFC licenses this file
-// to you under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// regarding copyright ownership.  The SFC licenses this file
+// to you under the Apache License, Version 2.0 (the
+// "License"); you may not use this file except in compliance
+// with the License.  You may obtain a copy of the License at
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
+//   http://www.apache.org/licenses/LICENSE-2.0
 //
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// Unless required by applicable law or agreed to in writing,
+// software distributed under the License is distributed on an
+// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+// KIND, either express or implied.  See the License for the
+// specific language governing permissions and limitations
+// under the License.
 // </copyright>
 
 using OpenQA.Selenium.Internal;
 using OpenQA.Selenium.Remote;
 using System;
 using System.Collections.Generic;
+using System.Diagnostics.CodeAnalysis;
 using System.Globalization;
-using System.Text.Json;
 
 namespace OpenQA.Selenium
 {
@@ -112,22 +113,9 @@ namespace OpenQA.Selenium
     /// </summary>
     public abstract class DriverOptions
     {
-        private string browserName;
-        private string browserVersion;
-        private string platformName;
-        private Proxy proxy;
-        private bool? acceptInsecureCertificates;
-        private bool? useWebSocketUrl;
-        private bool useStrictFileInteractability;
-        private bool? enableDownloads;
-        private TimeSpan? scriptTimeout;
-        private TimeSpan? pageLoadTimeout;
-        private TimeSpan? implicitWaitTimeout;
-        private UnhandledPromptBehavior unhandledPromptBehavior = UnhandledPromptBehavior.Default;
-        private PageLoadStrategy pageLoadStrategy = PageLoadStrategy.Default;
-        private Dictionary<string, object> additionalCapabilities = new Dictionary<string, object>();
-        private Dictionary<string, LogLevel> loggingPreferences = new Dictionary<string, LogLevel>();
-        private Dictionary<string, string> knownCapabilityNames = new Dictionary<string, string>();
+        private readonly Dictionary<string, object> additionalCapabilities = new Dictionary<string, object>();
+        private readonly Dictionary<string, LogLevel> loggingPreferences = new Dictionary<string, LogLevel>();
+        private readonly Dictionary<string, string> knownCapabilityNames = new Dictionary<string, string>();
 
         /// <summary>
         /// Initializes a new instance of the <see cref="DriverOptions"/> class.
@@ -148,97 +136,57 @@ namespace OpenQA.Selenium
         /// <summary>
         /// Gets or sets the name of the browser.
         /// </summary>
-        public string BrowserName
-        {
-            get { return this.browserName; }
-            protected set { this.browserName = value; }
-        }
+        public string? BrowserName { get; protected set; }
 
         /// <summary>
         /// Gets or sets the version of the browser.
         /// </summary>
-        public string BrowserVersion
-        {
-            get { return this.browserVersion; }
-            set { this.browserVersion = value; }
-        }
+        public string? BrowserVersion { get; set; }
 
         /// <summary>
         /// Gets or sets the name of the platform on which the browser is running.
         /// </summary>
-        public string PlatformName
-        {
-            get { return this.platformName; }
-            set { this.platformName = value; }
-        }
+        public string? PlatformName { get; set; }
 
         /// <summary>
         /// Gets or sets a value indicating whether the browser should accept self-signed
         /// SSL certificates.
         /// </summary>
-        public bool? AcceptInsecureCertificates
-        {
-            get { return this.acceptInsecureCertificates; }
-            set { this.acceptInsecureCertificates = value; }
-        }
+        public bool? AcceptInsecureCertificates { get; set; }
 
         /// <summary>
         /// Gets or sets a value indicating whether the driver should request a URL to
         /// a WebSocket to be used for bidirectional communication.
         /// </summary>
-        public bool? UseWebSocketUrl
-        {
-            get { return this.useWebSocketUrl; }
-            set { this.useWebSocketUrl = value; }
-        }
+        public bool? UseWebSocketUrl { get; set; }
 
         /// <summary>
         /// Gets or sets the value for describing how unexpected alerts are to be handled in the browser.
         /// Defaults to <see cref="UnhandledPromptBehavior.Default"/>.
         /// </summary>
-        public UnhandledPromptBehavior UnhandledPromptBehavior
-        {
-            get { return this.unhandledPromptBehavior; }
-            set { this.unhandledPromptBehavior = value; }
-        }
+        public UnhandledPromptBehavior UnhandledPromptBehavior { get; set; } = UnhandledPromptBehavior.Default;
 
         /// <summary>
         /// Gets or sets the value for describing how the browser is to wait for pages to load in the browser.
         /// Defaults to <see cref="PageLoadStrategy.Default"/>.
         /// </summary>
-        public PageLoadStrategy PageLoadStrategy
-        {
-            get { return this.pageLoadStrategy; }
-            set { this.pageLoadStrategy = value; }
-        }
+        public PageLoadStrategy PageLoadStrategy { get; set; } = PageLoadStrategy.Default;
 
         /// <summary>
         /// Gets or sets the <see cref="Proxy"/> to be used with this browser.
         /// </summary>
-        public Proxy Proxy
-        {
-            get { return this.proxy; }
-            set { this.proxy = value; }
-        }
+        public Proxy? Proxy { get; set; }
 
         /// <summary>
         /// Gets or sets a value indicating whether &lt;input type='file'/&gt; elements
         /// must be visible to allow uploading of files.
         /// </summary>
-        public bool UseStrictFileInteractability
-        {
-            get { return this.useStrictFileInteractability; }
-            set { this.useStrictFileInteractability = value; }
-        }
+        public bool UseStrictFileInteractability { get; set; }
 
         /// <summary>
         /// Gets or sets a value indicating whether files may be downloaded from remote node.
         /// </summary>
-        public bool? EnableDownloads
-        {
-            get { return this.enableDownloads; }
-            set { this.enableDownloads = value; }
-        }
+        public bool? EnableDownloads { get; set; }
 
         /// <summary>
         /// Gets or sets the asynchronous script timeout, which is the amount
@@ -246,22 +194,14 @@ namespace OpenQA.Selenium
         /// This timeout only affects the <see cref="IJavaScriptExecutor.ExecuteAsyncScript(string, object[])"/>
         /// method.
         /// </summary>
-        public TimeSpan? ScriptTimeout
-        {
-            get { return this.scriptTimeout; }
-            set { this.scriptTimeout = value; }
-        }
+        public TimeSpan? ScriptTimeout { get; set; }
 
         /// <summary>
         /// Gets or sets the page load timeout, which is the amount of time the driver
         /// should wait for a page to load when setting the <see cref="IWebDriver.Url"/>
         /// property.
         /// </summary>
-        public TimeSpan? PageLoadTimeout
-        {
-            get { return this.pageLoadTimeout; }
-            set { this.pageLoadTimeout = value; }
-        }
+        public TimeSpan? PageLoadTimeout { get; set; }
 
         /// <summary>
         /// Gets or sets the implicit wait timeout, which is the  amount of time the
@@ -280,20 +220,16 @@ namespace OpenQA.Selenium
         /// slower location strategies like XPath.
         /// </para>
         /// </remarks>
-        public TimeSpan? ImplicitWaitTimeout
-        {
-            get { return this.implicitWaitTimeout; }
-            set { this.implicitWaitTimeout = value; }
-        }
+        public TimeSpan? ImplicitWaitTimeout { get; set; }
 
         /// <summary>
         /// Set or Get the location of the browser
         /// Override in subclass
         /// </summary>
-        public virtual string BinaryLocation
+        public virtual string? BinaryLocation
         {
-            get { return null; }
-            set { throw new NotImplementedException(); }
+            get => null;
+            set => throw new NotImplementedException();
         }
 
         /// <summary>
@@ -330,45 +266,51 @@ namespace OpenQA.Selenium
         /// </summary>
         /// <param name="other">The <see cref="DriverOptions"/> object to compare with.</param>
         /// <returns>A <see cref="DriverOptionsMergeResult"/> object containing the status of the attempted merge.</returns>
+        /// <exception cref="ArgumentNullException">If <paramref name="other"/> is <see langword="null"/>.</exception>
         public virtual DriverOptionsMergeResult GetMergeResult(DriverOptions other)
         {
+            if (other is null)
+            {
+                throw new ArgumentNullException(nameof(other));
+            }
+
             DriverOptionsMergeResult result = new DriverOptionsMergeResult();
-            if (this.browserName != null && other.BrowserName != null)
+            if (this.BrowserName != null && other.BrowserName != null)
             {
                 result.IsMergeConflict = true;
                 result.MergeConflictOptionName = "BrowserName";
                 return result;
             }
 
-            if (this.browserVersion != null && other.BrowserVersion != null)
+            if (this.BrowserVersion != null && other.BrowserVersion != null)
             {
                 result.IsMergeConflict = true;
                 result.MergeConflictOptionName = "BrowserVersion";
                 return result;
             }
 
-            if (this.platformName != null && other.PlatformName != null)
+            if (this.PlatformName != null && other.PlatformName != null)
             {
                 result.IsMergeConflict = true;
                 result.MergeConflictOptionName = "PlatformName";
                 return result;
             }
 
-            if (this.proxy != null && other.Proxy != null)
+            if (this.Proxy != null && other.Proxy != null)
             {
                 result.IsMergeConflict = true;
                 result.MergeConflictOptionName = "Proxy";
                 return result;
             }
 
-            if (this.unhandledPromptBehavior != UnhandledPromptBehavior.Default && other.UnhandledPromptBehavior != UnhandledPromptBehavior.Default)
+            if (this.UnhandledPromptBehavior != UnhandledPromptBehavior.Default && other.UnhandledPromptBehavior != UnhandledPromptBehavior.Default)
             {
                 result.IsMergeConflict = true;
                 result.MergeConflictOptionName = "UnhandledPromptBehavior";
                 return result;
             }
 
-            if (this.pageLoadStrategy != PageLoadStrategy.Default && other.PageLoadStrategy != PageLoadStrategy.Default)
+            if (this.PageLoadStrategy != PageLoadStrategy.Default && other.PageLoadStrategy != PageLoadStrategy.Default)
             {
                 result.IsMergeConflict = true;
                 result.MergeConflictOptionName = "PageLoadStrategy";
@@ -389,23 +331,14 @@ namespace OpenQA.Selenium
             this.loggingPreferences[logType] = logLevel;
         }
 
-        /// <summary>
-        /// Returns a string representation of this <see cref="DriverOptions"/>.
-        /// </summary>
-        /// <returns>A string representation of this <see cref="DriverOptions"/>.</returns>
-        public override string ToString()
-        {
-            return JsonSerializer.Serialize(this.ToDictionary(), new JsonSerializerOptions { WriteIndented = true });
-        }
-
         /// <summary>
         /// Returns the current options as a <see cref="Dictionary{TKey, TValue}"/>.
         /// </summary>
         /// <returns>The current options as a <see cref="Dictionary{TKey, TValue}"/>.</returns>
-        internal IDictionary<string, object> ToDictionary()
+        internal IDictionary<string, object>? ToDictionary()
         {
-            IHasCapabilitiesDictionary desired = this.ToCapabilities() as IHasCapabilitiesDictionary;
-            if (desired == null)
+            ICapabilities? capabilities = this.ToCapabilities();
+            if (capabilities is not IHasCapabilitiesDictionary desired)
             {
                 return null;
             }
@@ -422,16 +355,15 @@ namespace OpenQA.Selenium
         /// thrown when attempting to add a capability for which there is already a type safe option, or
         /// when <paramref name="capabilityName"/> is <see langword="null"/> or the empty string.
         /// </exception>
-        protected void ValidateCapabilityName(string capabilityName)
+        protected void ValidateCapabilityName([NotNull] string? capabilityName)
         {
-            if (string.IsNullOrEmpty(capabilityName))
+            if (capabilityName is null || string.IsNullOrEmpty(capabilityName))
             {
                 throw new ArgumentException("Capability name may not be null an empty string.", nameof(capabilityName));
             }
 
-            if (this.IsKnownCapabilityName(capabilityName))
+            if (this.TryGetKnownCapability(capabilityName!, out string? typeSafeOptionName))
             {
-                string typeSafeOptionName = this.GetTypeSafeOptionName(capabilityName);
                 string message = string.Format(CultureInfo.InvariantCulture, "There is already an option for the {0} capability. Please use the {1} instead.", capabilityName, typeSafeOptionName);
                 throw new ArgumentException(message, nameof(capabilityName));
             }
@@ -452,9 +384,9 @@ namespace OpenQA.Selenium
         /// Remove a capability from the list of known capabilities
         /// </summary>
         /// <param name="capabilityName">The name of the capability to be removed.</param>
-        protected void RemoveKnownCapabilityName(string capabilityName)
+        protected void RemoveKnownCapabilityName(string? capabilityName)
         {
-            if (!string.IsNullOrEmpty(capabilityName) && this.knownCapabilityNames.ContainsKey(capabilityName))
+            if (capabilityName is not null)
             {
                 this.knownCapabilityNames.Remove(capabilityName);
             }
@@ -470,6 +402,17 @@ namespace OpenQA.Selenium
             return this.knownCapabilityNames.ContainsKey(capabilityName);
         }
 
+        /// <summary>
+        /// Gets a value indicating whether the specified capability name is a known capability name which has a type-safe option.
+        /// </summary>
+        /// <param name="capabilityName">The name of the capability to check.</param>
+        /// <param name="typeSafeOptionName">The name of the type-safe option for the given capability name, or <see langword="null"/> if not found.</param>
+        /// <returns><see langword="true"/> if the capability name is known; otherwise <see langword="false"/>.</returns>
+        protected bool TryGetKnownCapability(string capabilityName, [NotNullWhen(true)] out string? typeSafeOptionName)
+        {
+            return this.knownCapabilityNames.TryGetValue(capabilityName, out typeSafeOptionName);
+        }
+
         /// <summary>
         /// Gets the name of the type-safe option for a given capability name.
         /// </summary>
@@ -489,7 +432,7 @@ namespace OpenQA.Selenium
         /// Generates the logging preferences dictionary for transmission as a desired capability.
         /// </summary>
         /// <returns>The dictionary containing the logging preferences.</returns>
-        protected Dictionary<string, object> GenerateLoggingPreferencesDictionary()
+        protected Dictionary<string, object>? GenerateLoggingPreferencesDictionary()
         {
             if (this.loggingPreferences.Count == 0)
             {
@@ -513,45 +456,45 @@ namespace OpenQA.Selenium
         protected IWritableCapabilities GenerateDesiredCapabilities(bool isSpecificationCompliant)
         {
             DesiredCapabilities capabilities = new DesiredCapabilities();
-            if (!string.IsNullOrEmpty(this.browserName))
+            if (!string.IsNullOrEmpty(this.BrowserName))
             {
-                capabilities.SetCapability(CapabilityType.BrowserName, this.browserName);
+                capabilities.SetCapability(CapabilityType.BrowserName, this.BrowserName!);
             }
 
-            if (!string.IsNullOrEmpty(this.browserVersion))
+            if (!string.IsNullOrEmpty(this.BrowserVersion))
             {
-                capabilities.SetCapability(CapabilityType.BrowserVersion, this.browserVersion);
+                capabilities.SetCapability(CapabilityType.BrowserVersion, this.BrowserVersion!);
             }
 
-            if (!string.IsNullOrEmpty(this.platformName))
+            if (!string.IsNullOrEmpty(this.PlatformName))
             {
-                capabilities.SetCapability(CapabilityType.PlatformName, this.platformName);
+                capabilities.SetCapability(CapabilityType.PlatformName, this.PlatformName!);
             }
 
-            if (this.acceptInsecureCertificates.HasValue)
+            if (this.AcceptInsecureCertificates.HasValue)
             {
-                capabilities.SetCapability(CapabilityType.AcceptInsecureCertificates, this.acceptInsecureCertificates);
+                capabilities.SetCapability(CapabilityType.AcceptInsecureCertificates, this.AcceptInsecureCertificates);
             }
 
-            if (this.useWebSocketUrl.HasValue)
+            if (this.UseWebSocketUrl.HasValue)
             {
-                capabilities.SetCapability(CapabilityType.WebSocketUrl, this.useWebSocketUrl);
+                capabilities.SetCapability(CapabilityType.WebSocketUrl, this.UseWebSocketUrl);
             }
 
-            if (this.enableDownloads.HasValue)
+            if (this.EnableDownloads.HasValue)
             {
-                capabilities.SetCapability(CapabilityType.EnableDownloads, this.enableDownloads);
+                capabilities.SetCapability(CapabilityType.EnableDownloads, this.EnableDownloads);
             }
 
-            if (this.useStrictFileInteractability)
+            if (this.UseStrictFileInteractability)
             {
                 capabilities.SetCapability(CapabilityType.UseStrictFileInteractability, true);
             }
 
-            if (this.pageLoadStrategy != PageLoadStrategy.Default)
+            if (this.PageLoadStrategy != PageLoadStrategy.Default)
             {
                 string pageLoadStrategySetting = "normal";
-                switch (this.pageLoadStrategy)
+                switch (this.PageLoadStrategy)
                 {
                     case PageLoadStrategy.Eager:
                         pageLoadStrategySetting = "eager";
@@ -592,7 +535,7 @@ namespace OpenQA.Selenium
 
             if (this.Proxy != null)
             {
-                Dictionary<string, object> proxyCapability = this.Proxy.ToCapability();
+                Dictionary<string, object?>? proxyCapability = this.Proxy.ToCapability();
                 if (!isSpecificationCompliant)
                 {
                     proxyCapability = this.Proxy.ToLegacyCapability();
@@ -604,13 +547,13 @@ namespace OpenQA.Selenium
                 }
             }
 
-            if (this.scriptTimeout.HasValue || this.pageLoadTimeout.HasValue || this.implicitWaitTimeout.HasValue)
+            if (this.ScriptTimeout.HasValue || this.PageLoadTimeout.HasValue || this.ImplicitWaitTimeout.HasValue)
             {
                 var timeouts = new Timeout
                 {
-                    Script = this.scriptTimeout,
-                    PageLoad = this.pageLoadTimeout,
-                    ImplicitWait = this.implicitWaitTimeout
+                    Script = this.ScriptTimeout,
+                    PageLoad = this.PageLoadTimeout,
+                    ImplicitWait = this.ImplicitWaitTimeout
                 };
 
                 capabilities.SetCapability(CapabilityType.Timeouts, timeouts.ToCapabilities());
