@@ -1,19 +1,20 @@
-// <copyright file="FirefoxOptions.cs" company="WebDriver Committers">
+// <copyright file="FirefoxOptions.cs" company="Selenium Committers">
 // Licensed to the Software Freedom Conservancy (SFC) under one
-// or more contributor license agreements. See the NOTICE file
+// or more contributor license agreements.  See the NOTICE file
 // distributed with this work for additional information
-// regarding copyright ownership. The SFC licenses this file
-// to you under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// regarding copyright ownership.  The SFC licenses this file
+// to you under the Apache License, Version 2.0 (the
+// "License"); you may not use this file except in compliance
+// with the License.  You may obtain a copy of the License at
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
+//   http://www.apache.org/licenses/LICENSE-2.0
 //
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// Unless required by applicable law or agreed to in writing,
+// software distributed under the License is distributed on an
+// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+// KIND, either express or implied.  See the License for the
+// specific language governing permissions and limitations
+// under the License.
 // </copyright>
 
 using System;
@@ -59,16 +60,10 @@ namespace OpenQA.Selenium.Firefox
         private const string FirefoxEnvCapability = "env";
         private const string FirefoxOptionsCapability = "moz:firefoxOptions";
         private const string FirefoxEnableDevToolsProtocolCapability = "moz:debuggerAddress";
-
-        private bool enableDevToolsProtocol;
-        private string binaryLocation;
-        private FirefoxDriverLogLevel logLevel = FirefoxDriverLogLevel.Default;
-        private FirefoxProfile profile;
-        private List<string> firefoxArguments = new List<string>();
-        private Dictionary<string, object> profilePreferences = new Dictionary<string, object>();
-        private Dictionary<string, object> additionalFirefoxOptions = new Dictionary<string, object>();
-        private Dictionary<string, object> environmentVariables = new Dictionary<string, object>();
-        private FirefoxAndroidOptions androidOptions;
+        private readonly List<string> firefoxArguments = new List<string>();
+        private readonly Dictionary<string, object> profilePreferences = new Dictionary<string, object>();
+        private readonly Dictionary<string, object> additionalFirefoxOptions = new Dictionary<string, object>();
+        private readonly Dictionary<string, object> environmentVariables = new Dictionary<string, object>();
 
         /// <summary>
         /// Initializes a new instance of the <see cref="FirefoxOptions"/> class.
@@ -96,63 +91,44 @@ namespace OpenQA.Selenium.Firefox
         /// <summary>
         /// Gets or sets the <see cref="FirefoxProfile"/> object to be used with this instance.
         /// </summary>
-        public FirefoxProfile Profile
-        {
-            get { return this.profile; }
-            set { this.profile = value; }
-        }
+        public FirefoxProfile? Profile { get; set; }
 
         /// <summary>
         /// Gets or sets the path and file name of the Firefox browser executable.
         /// </summary>
-        public override string BinaryLocation
-        {
-            get { return this.binaryLocation; }
-            set { this.binaryLocation = value; }
-        }
+        public override string? BinaryLocation { get; set; }
 
         /// <summary>
         /// Gets or sets the path and file name of the Firefox browser executable.
         /// </summary>
         [Obsolete("Use BinaryLocation property instead of BrowserExecutableLocation. This one will be removed soon.")]
-        public string BrowserExecutableLocation
+        public string? BrowserExecutableLocation
         {
-            get { return this.binaryLocation; }
-            set { this.binaryLocation = value; }
+            get => this.BinaryLocation;
+            set => this.BinaryLocation = value;
         }
 
         /// <summary>
         /// Gets or sets the logging level of the Firefox driver.
         /// </summary>
-        public FirefoxDriverLogLevel LogLevel
-        {
-            get { return this.logLevel; }
-            set { this.logLevel = value; }
-        }
+        public FirefoxDriverLogLevel LogLevel { get; set; } = FirefoxDriverLogLevel.Default;
 
         /// <summary>
         /// Gets or sets a value indicating whether to enable the DevTools protocol for the launched browser.
         /// </summary>
-        public bool EnableDevToolsProtocol
-        {
-            get { return this.enableDevToolsProtocol; }
-            set { this.enableDevToolsProtocol = value; }
-        }
+        public bool EnableDevToolsProtocol { get; set; }
 
         /// <summary>
         /// Gets or sets the options for automating Firefox on Android.
         /// </summary>
-        public FirefoxAndroidOptions AndroidOptions
-        {
-            get { return this.androidOptions; }
-            set { this.androidOptions = value; }
-        }
+        public FirefoxAndroidOptions? AndroidOptions { get; set; }
 
         /// <summary>
         /// Adds an argument to be used in launching the Firefox browser.
         /// </summary>
         /// <param name="argumentName">The argument to add.</param>
-        /// <remarks>Arguments must be preceeded by two dashes ("--").</remarks>
+        /// <remarks>Arguments must be preceded by two dashes ("--").</remarks>
+        /// <exception cref="ArgumentException">If <paramref name="argumentName"/> is <see langword="null"/> or <see cref="string.Empty"/>.</exception>
         public void AddArgument(string argumentName)
         {
             if (string.IsNullOrEmpty(argumentName))
@@ -167,16 +143,18 @@ namespace OpenQA.Selenium.Firefox
         /// Adds a list arguments to be used in launching the Firefox browser.
         /// </summary>
         /// <param name="argumentsToAdd">An array of arguments to add.</param>
-        /// <remarks>Each argument must be preceeded by two dashes ("--").</remarks>
+        /// <remarks>Each argument must be preceded by two dashes ("--").</remarks>
+        /// <exception cref="ArgumentNullException">If <paramref name="argumentsToAdd"/> is <see langword="null"/>.</exception>
         public void AddArguments(params string[] argumentsToAdd)
         {
-            this.AddArguments(new List<string>(argumentsToAdd));
+            this.AddArguments((IEnumerable<string>)argumentsToAdd);
         }
 
         /// <summary>
         /// Adds a list arguments to be used in launching the Firefox browser.
         /// </summary>
         /// <param name="argumentsToAdd">An array of arguments to add.</param>
+        /// <exception cref="ArgumentNullException">If <paramref name="argumentsToAdd"/> is <see langword="null"/>.</exception>
         public void AddArguments(IEnumerable<string> argumentsToAdd)
         {
             if (argumentsToAdd == null)
@@ -192,6 +170,7 @@ namespace OpenQA.Selenium.Firefox
         /// </summary>
         /// <param name="preferenceName">Name of the preference to set.</param>
         /// <param name="preferenceValue">Value of the preference to set.</param>
+        /// <exception cref="ArgumentException">If <paramref name="preferenceName"/> is <see langword="null"/> or <see cref="string.Empty"/>.</exception>
         public void SetPreference(string preferenceName, bool preferenceValue)
         {
             this.SetPreferenceValue(preferenceName, preferenceValue);
@@ -202,6 +181,7 @@ namespace OpenQA.Selenium.Firefox
         /// </summary>
         /// <param name="preferenceName">Name of the preference to set.</param>
         /// <param name="preferenceValue">Value of the preference to set.</param>
+        /// <exception cref="ArgumentException">If <paramref name="preferenceName"/> is <see langword="null"/> or <see cref="string.Empty"/>.</exception>
         public void SetPreference(string preferenceName, int preferenceValue)
         {
             this.SetPreferenceValue(preferenceName, preferenceValue);
@@ -212,6 +192,7 @@ namespace OpenQA.Selenium.Firefox
         /// </summary>
         /// <param name="preferenceName">Name of the preference to set.</param>
         /// <param name="preferenceValue">Value of the preference to set.</param>
+        /// <exception cref="ArgumentException">If <paramref name="preferenceName"/> is <see langword="null"/> or <see cref="string.Empty"/>.</exception>
         public void SetPreference(string preferenceName, long preferenceValue)
         {
             this.SetPreferenceValue(preferenceName, preferenceValue);
@@ -222,6 +203,7 @@ namespace OpenQA.Selenium.Firefox
         /// </summary>
         /// <param name="preferenceName">Name of the preference to set.</param>
         /// <param name="preferenceValue">Value of the preference to set.</param>
+        /// <exception cref="ArgumentException">If <paramref name="preferenceName"/> is <see langword="null"/> or <see cref="string.Empty"/>.</exception>
         public void SetPreference(string preferenceName, double preferenceValue)
         {
             this.SetPreferenceValue(preferenceName, preferenceValue);
@@ -232,29 +214,26 @@ namespace OpenQA.Selenium.Firefox
         /// </summary>
         /// <param name="preferenceName">Name of the preference to set.</param>
         /// <param name="preferenceValue">Value of the preference to set.</param>
+        /// <exception cref="ArgumentException">If <paramref name="preferenceName"/> is <see langword="null"/> or <see cref="string.Empty"/>.</exception>
         public void SetPreference(string preferenceName, string preferenceValue)
         {
             this.SetPreferenceValue(preferenceName, preferenceValue);
         }
 
         /// <summary>
-        /// Sets an environment variable to be set in the operating system's environment under which the Firerox browser is launched.
+        /// Sets an environment variable to be set in the operating system's environment under which the Firefox browser is launched.
         /// </summary>
         /// <param name="variableName">The name of the environment variable.</param>
         /// <param name="variableValue">The value of the environment variable.</param>
-        public void SetEnvironmentVariable(string variableName, string variableValue)
+        /// <exception cref="ArgumentException">If <paramref name="variableName"/> is <see langword="null"/> or <see cref="string.Empty"/>.</exception>
+        public void SetEnvironmentVariable(string variableName, string? variableValue)
         {
             if (string.IsNullOrEmpty(variableName))
             {
                 throw new ArgumentException("Environment variable name cannot be null or an empty string");
             }
 
-            if (variableValue == null)
-            {
-                variableValue = string.Empty;
-            }
-
-            this.environmentVariables[variableName] = variableValue;
+            this.environmentVariables[variableName] = variableValue ?? string.Empty;
         }
 
         /// <summary>
@@ -289,7 +268,7 @@ namespace OpenQA.Selenium.Firefox
             IWritableCapabilities capabilities = GenerateDesiredCapabilities(true);
             Dictionary<string, object> firefoxOptions = this.GenerateFirefoxOptionsDictionary();
             capabilities.SetCapability(FirefoxOptionsCapability, firefoxOptions);
-            if (this.enableDevToolsProtocol)
+            if (this.EnableDevToolsProtocol)
             {
                 capabilities.SetCapability(FirefoxEnableDevToolsProtocolCapability, true);
             }
@@ -301,30 +280,26 @@ namespace OpenQA.Selenium.Firefox
         {
             Dictionary<string, object> firefoxOptions = new Dictionary<string, object>();
 
-            if (this.profile != null)
+            if (this.Profile != null)
             {
-                firefoxOptions[FirefoxProfileCapability] = this.profile.ToBase64String();
+                firefoxOptions[FirefoxProfileCapability] = this.Profile.ToBase64String();
             }
 
-            if (!string.IsNullOrEmpty(this.binaryLocation))
+            if (!string.IsNullOrEmpty(this.BinaryLocation))
             {
-                firefoxOptions[FirefoxBinaryCapability] = this.binaryLocation;
+                firefoxOptions[FirefoxBinaryCapability] = this.BinaryLocation!;
             }
 
-            if (this.logLevel != FirefoxDriverLogLevel.Default)
+            if (this.LogLevel != FirefoxDriverLogLevel.Default)
             {
                 Dictionary<string, object> logObject = new Dictionary<string, object>();
-                logObject["level"] = this.logLevel.ToString().ToLowerInvariant();
+                logObject["level"] = this.LogLevel.ToString().ToLowerInvariant();
                 firefoxOptions[FirefoxLogCapability] = logObject;
             }
 
             if (this.firefoxArguments.Count > 0)
             {
-                List<object> args = new List<object>();
-                foreach (string argument in this.firefoxArguments)
-                {
-                    args.Add(argument);
-                }
+                List<object> args = [.. this.firefoxArguments];
 
                 firefoxOptions[FirefoxArgumentsCapability] = args;
             }
@@ -339,9 +314,9 @@ namespace OpenQA.Selenium.Firefox
                 firefoxOptions[FirefoxEnvCapability] = this.environmentVariables;
             }
 
-            if (this.androidOptions != null)
+            if (this.AndroidOptions != null)
             {
-                this.AddAndroidOptions(firefoxOptions);
+                AddAndroidOptions(this.AndroidOptions, firefoxOptions);
             }
 
             foreach (KeyValuePair<string, object> pair in this.additionalFirefoxOptions)
@@ -362,27 +337,23 @@ namespace OpenQA.Selenium.Firefox
             this.profilePreferences[preferenceName] = preferenceValue;
         }
 
-        private void AddAndroidOptions(Dictionary<string, object> firefoxOptions)
+        private static void AddAndroidOptions(FirefoxAndroidOptions androidOptions, Dictionary<string, object> firefoxOptions)
         {
-            firefoxOptions["androidPackage"] = this.androidOptions.AndroidPackage;
+            firefoxOptions["androidPackage"] = androidOptions.AndroidPackage;
 
-            if (!string.IsNullOrEmpty(this.androidOptions.AndroidDeviceSerial))
+            if (!string.IsNullOrEmpty(androidOptions.AndroidDeviceSerial))
             {
-                firefoxOptions["androidDeviceSerial"] = this.androidOptions.AndroidDeviceSerial;
+                firefoxOptions["androidDeviceSerial"] = androidOptions.AndroidDeviceSerial!;
             }
 
-            if (!string.IsNullOrEmpty(this.androidOptions.AndroidActivity))
+            if (!string.IsNullOrEmpty(androidOptions.AndroidActivity))
             {
-                firefoxOptions["androidActivity"] = this.androidOptions.AndroidActivity;
+                firefoxOptions["androidActivity"] = androidOptions.AndroidActivity!;
             }
 
-            if (this.androidOptions.AndroidIntentArguments.Count > 0)
+            if (androidOptions.AndroidIntentArguments.Count > 0)
             {
-                List<object> args = new List<object>();
-                foreach (string argument in this.androidOptions.AndroidIntentArguments)
-                {
-                    args.Add(argument);
-                }
+                List<object> args = [.. androidOptions.AndroidIntentArguments];
 
                 firefoxOptions["androidIntentArguments"] = args;
             }
