@@ -1,19 +1,20 @@
-// <copyright file="Proxy.cs" company="WebDriver Committers">
+// <copyright file="Proxy.cs" company="Selenium Committers">
 // Licensed to the Software Freedom Conservancy (SFC) under one
-// or more contributor license agreements. See the NOTICE file
+// or more contributor license agreements.  See the NOTICE file
 // distributed with this work for additional information
-// regarding copyright ownership. The SFC licenses this file
-// to you under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// regarding copyright ownership.  The SFC licenses this file
+// to you under the Apache License, Version 2.0 (the
+// "License"); you may not use this file except in compliance
+// with the License.  You may obtain a copy of the License at
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
+//   http://www.apache.org/licenses/LICENSE-2.0
 //
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// Unless required by applicable law or agreed to in writing,
+// software distributed under the License is distributed on an
+// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+// KIND, either express or implied.  See the License for the
+// specific language governing permissions and limitations
+// under the License.
 // </copyright>
 
 using System;
@@ -71,13 +72,13 @@ namespace OpenQA.Selenium
     {
         private ProxyKind proxyKind = ProxyKind.Unspecified;
         private bool isAutoDetect;
-        private string ftpProxyLocation;
-        private string httpProxyLocation;
-        private string proxyAutoConfigUrl;
-        private string sslProxyLocation;
-        private string socksProxyLocation;
-        private string socksUserName;
-        private string socksPassword;
+        private string? ftpProxyLocation;
+        private string? httpProxyLocation;
+        private string? proxyAutoConfigUrl;
+        private string? sslProxyLocation;
+        private string? socksProxyLocation;
+        private string? socksUserName;
+        private string? socksPassword;
         private int? socksVersion;
         private List<string> noProxyAddresses = new List<string>();
 
@@ -92,6 +93,8 @@ namespace OpenQA.Selenium
         /// Initializes a new instance of the <see cref="Proxy"/> class with the given proxy settings.
         /// </summary>
         /// <param name="settings">A dictionary of settings to use with the proxy.</param>
+        /// <exception cref="ArgumentNullException">If <paramref name="settings"/> is <see langword="null"/>.</exception>
+        /// <exception cref="ArgumentException">If The "noProxy" value is a list with a <see langword="null"/> element.</exception>
         public Proxy(Dictionary<string, object> settings)
         {
             if (settings == null)
@@ -99,46 +102,44 @@ namespace OpenQA.Selenium
                 throw new ArgumentNullException(nameof(settings), "settings dictionary cannot be null");
             }
 
-            if (settings.ContainsKey("proxyType") && settings["proxyType"] != null)
+            if (settings.TryGetValue("proxyType", out object? proxyTypeObj) && proxyTypeObj?.ToString() is string proxyType)
             {
                 // Special-case "PAC" since that is the correct serialization.
-                if (settings["proxyType"].ToString().ToLowerInvariant() == "pac")
+                if (proxyType.Equals("pac", StringComparison.InvariantCultureIgnoreCase))
                 {
                     this.Kind = ProxyKind.ProxyAutoConfigure;
                 }
                 else
                 {
-                    ProxyKind rawType = (ProxyKind)Enum.Parse(typeof(ProxyKind), settings["proxyType"].ToString(), true);
+                    ProxyKind rawType = (ProxyKind)Enum.Parse(typeof(ProxyKind), proxyType, ignoreCase: true);
                     this.Kind = rawType;
                 }
             }
 
-            if (settings.ContainsKey("ftpProxy") && settings["ftpProxy"] != null)
+            if (settings.TryGetValue("ftpProxy", out object? ftpProxyObj) && ftpProxyObj?.ToString() is string ftpProxy)
             {
-                this.FtpProxy = settings["ftpProxy"].ToString();
+                this.FtpProxy = ftpProxy;
             }
 
-            if (settings.ContainsKey("httpProxy") && settings["httpProxy"] != null)
+            if (settings.TryGetValue("httpProxy", out object? httpProxyObj) && httpProxyObj?.ToString() is string httpProxy)
             {
-                this.HttpProxy = settings["httpProxy"].ToString();
+                this.HttpProxy = httpProxy;
             }
 
-            if (settings.ContainsKey("noProxy") && settings["noProxy"] != null)
+            if (settings.TryGetValue("noProxy", out object? noProxy) && noProxy != null)
             {
                 List<string> bypassAddresses = new List<string>();
-                string addressesAsString = settings["noProxy"] as string;
-                if (addressesAsString != null)
+                if (noProxy is string addressesAsString)
                 {
                     bypassAddresses.AddRange(addressesAsString.Split(';'));
                 }
                 else
                 {
-                    object[] addressesAsArray = settings["noProxy"] as object[];
-                    if (addressesAsArray != null)
+                    if (noProxy is object?[] addressesAsArray)
                     {
-                        foreach (object address in addressesAsArray)
+                        foreach (object? address in addressesAsArray)
                         {
-                            bypassAddresses.Add(address.ToString());
+                            bypassAddresses.Add(address?.ToString() ?? throw new ArgumentException("Proxy bypass address list \"noProxy\" contained a null element", nameof(settings)));
                         }
                     }
                 }
@@ -146,39 +147,39 @@ namespace OpenQA.Selenium
                 this.AddBypassAddresses(bypassAddresses);
             }
 
-            if (settings.ContainsKey("proxyAutoconfigUrl") && settings["proxyAutoconfigUrl"] != null)
+            if (settings.TryGetValue("proxyAutoconfigUrl", out object? proxyAutoconfigUrlObj) && proxyAutoconfigUrlObj?.ToString() is string proxyAutoconfigUrl)
             {
-                this.ProxyAutoConfigUrl = settings["proxyAutoconfigUrl"].ToString();
+                this.ProxyAutoConfigUrl = proxyAutoconfigUrl;
             }
 
-            if (settings.ContainsKey("sslProxy") && settings["sslProxy"] != null)
+            if (settings.TryGetValue("sslProxy", out object? sslProxyObj) && sslProxyObj?.ToString() is string sslProxy)
             {
-                this.SslProxy = settings["sslProxy"].ToString();
+                this.SslProxy = sslProxy;
             }
 
-            if (settings.ContainsKey("socksProxy") && settings["socksProxy"] != null)
+            if (settings.TryGetValue("socksProxy", out object? socksProxyObj) && socksProxyObj?.ToString() is string socksProxy)
             {
-                this.SocksProxy = settings["socksProxy"].ToString();
+                this.SocksProxy = socksProxy;
             }
 
-            if (settings.ContainsKey("socksUsername") && settings["socksUsername"] != null)
+            if (settings.TryGetValue("socksUsername", out object? socksUsernameObj) && socksUsernameObj?.ToString() is string socksUsername)
             {
-                this.SocksUserName = settings["socksUsername"].ToString();
+                this.SocksUserName = socksUsername;
             }
 
-            if (settings.ContainsKey("socksPassword") && settings["socksPassword"] != null)
+            if (settings.TryGetValue("socksPassword", out object? socksPasswordObj) && socksPasswordObj?.ToString() is string socksPassword)
             {
-                this.SocksPassword = settings["socksPassword"].ToString();
+                this.SocksPassword = socksPassword;
             }
 
-            if (settings.ContainsKey("socksVersion") && settings["socksVersion"] != null)
+            if (settings.TryGetValue("socksVersion", out object? socksVersion) && socksVersion != null)
             {
-                this.SocksVersion = Convert.ToInt32(settings["socksVersion"]);
+                this.SocksVersion = Convert.ToInt32(socksVersion);
             }
 
-            if (settings.ContainsKey("autodetect") && settings["autodetect"] != null)
+            if (settings.TryGetValue("autodetect", out object? autodetect) && autodetect != null)
             {
-                this.IsAutoDetect = (bool)settings["autodetect"];
+                this.IsAutoDetect = Convert.ToBoolean(autodetect);
             }
         }
 
@@ -188,10 +189,7 @@ namespace OpenQA.Selenium
         [JsonIgnore]
         public ProxyKind Kind
         {
-            get
-            {
-                return this.proxyKind;
-            }
+            get => this.proxyKind;
 
             set
             {
@@ -223,10 +221,7 @@ namespace OpenQA.Selenium
         [JsonIgnore]
         public bool IsAutoDetect
         {
-            get
-            {
-                return this.isAutoDetect;
-            }
+            get => this.isAutoDetect;
 
             set
             {
@@ -246,12 +241,9 @@ namespace OpenQA.Selenium
         /// </summary>
         [JsonPropertyName("ftpProxy")]
         [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingDefault)]
-        public string FtpProxy
+        public string? FtpProxy
         {
-            get
-            {
-                return this.ftpProxyLocation;
-            }
+            get => this.ftpProxyLocation;
 
             set
             {
@@ -266,12 +258,9 @@ namespace OpenQA.Selenium
         /// </summary>
         [JsonPropertyName("httpProxy")]
         [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingDefault)]
-        public string HttpProxy
+        public string? HttpProxy
         {
-            get
-            {
-                return this.httpProxyLocation;
-            }
+            get => this.httpProxyLocation;
 
             set
             {
@@ -286,7 +275,7 @@ namespace OpenQA.Selenium
         /// </summary>
         [JsonPropertyName("noProxy")]
         [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingDefault)]
-        public ReadOnlyCollection<string> BypassProxyAddresses
+        public ReadOnlyCollection<string>? BypassProxyAddresses
         {
             get
             {
@@ -304,12 +293,9 @@ namespace OpenQA.Selenium
         /// </summary>
         [JsonPropertyName("proxyAutoconfigUrl")]
         [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingDefault)]
-        public string ProxyAutoConfigUrl
+        public string? ProxyAutoConfigUrl
         {
-            get
-            {
-                return this.proxyAutoConfigUrl;
-            }
+            get => this.proxyAutoConfigUrl;
 
             set
             {
@@ -324,12 +310,9 @@ namespace OpenQA.Selenium
         /// </summary>
         [JsonPropertyName("sslProxy")]
         [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingDefault)]
-        public string SslProxy
+        public string? SslProxy
         {
-            get
-            {
-                return this.sslProxyLocation;
-            }
+            get => this.sslProxyLocation;
 
             set
             {
@@ -344,12 +327,9 @@ namespace OpenQA.Selenium
         /// </summary>
         [JsonPropertyName("socksProxy")]
         [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingDefault)]
-        public string SocksProxy
+        public string? SocksProxy
         {
-            get
-            {
-                return this.socksProxyLocation;
-            }
+            get => this.socksProxyLocation;
 
             set
             {
@@ -364,12 +344,9 @@ namespace OpenQA.Selenium
         /// </summary>
         [JsonPropertyName("socksUsername")]
         [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingDefault)]
-        public string SocksUserName
+        public string? SocksUserName
         {
-            get
-            {
-                return this.socksUserName;
-            }
+            get => this.socksUserName;
 
             set
             {
@@ -387,10 +364,7 @@ namespace OpenQA.Selenium
         [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingDefault)]
         public int? SocksVersion
         {
-            get
-            {
-                return this.socksVersion;
-            }
+            get => this.socksVersion;
 
             set
             {
@@ -402,7 +376,7 @@ namespace OpenQA.Selenium
                 {
                     if (value.Value <= 0)
                     {
-                        throw new ArgumentException("SocksVersion must be a positive integer");
+                        throw new ArgumentOutOfRangeException(nameof(value), "SocksVersion must be a positive integer");
                     }
 
                     this.VerifyProxyTypeCompatilibily(ProxyKind.Manual);
@@ -417,12 +391,9 @@ namespace OpenQA.Selenium
         /// </summary>
         [JsonPropertyName("socksPassword")]
         [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingDefault)]
-        public string SocksPassword
+        public string? SocksPassword
         {
-            get
-            {
-                return this.socksPassword;
-            }
+            get => this.socksPassword;
 
             set
             {
@@ -452,7 +423,7 @@ namespace OpenQA.Selenium
         /// <param name="addressesToAdd">An array of addresses to add.</param>
         public void AddBypassAddresses(params string[] addressesToAdd)
         {
-            this.AddBypassAddresses(new List<string>(addressesToAdd));
+            this.AddBypassAddresses((IEnumerable<string>)addressesToAdd);
         }
 
         /// <summary>
@@ -477,7 +448,7 @@ namespace OpenQA.Selenium
         /// </summary>
         /// <returns>A dictionary suitable for serializing to the W3C Specification
         /// dialect of the wire protocol.</returns>
-        internal Dictionary<string, object> ToCapability()
+        internal Dictionary<string, object?>? ToCapability()
         {
             return this.AsDictionary(true);
         }
@@ -488,17 +459,17 @@ namespace OpenQA.Selenium
         /// </summary>
         /// <returns>A dictionary suitable for serializing to the OSS dialect of the
         /// wire protocol.</returns>
-        internal Dictionary<string, object> ToLegacyCapability()
+        internal Dictionary<string, object?>? ToLegacyCapability()
         {
             return this.AsDictionary(false);
         }
 
-        private Dictionary<string, object> AsDictionary(bool isSpecCompliant)
+        private Dictionary<string, object?>? AsDictionary(bool isSpecCompliant)
         {
-            Dictionary<string, object> serializedDictionary = null;
+            Dictionary<string, object?>? serializedDictionary = null;
             if (this.proxyKind != ProxyKind.Unspecified)
             {
-                serializedDictionary = new Dictionary<string, object>();
+                serializedDictionary = new Dictionary<string, object?>();
                 if (this.proxyKind == ProxyKind.ProxyAutoConfigure)
                 {
                     serializedDictionary["proxyType"] = "pac";
@@ -555,17 +526,12 @@ namespace OpenQA.Selenium
             return serializedDictionary;
         }
 
-        private object GetNoProxyAddressList(bool isSpecCompliant)
+        private object? GetNoProxyAddressList(bool isSpecCompliant)
         {
-            object addresses = null;
+            object? addresses = null;
             if (isSpecCompliant)
             {
-                List<object> addressList = new List<object>();
-                foreach (string address in this.noProxyAddresses)
-                {
-                    addressList.Add(address);
-                }
-
+                List<object> addressList = [.. this.noProxyAddresses];
                 addresses = addressList;
             }
             else
