@@ -1,3 +1,22 @@
+// <copyright file="FormHandlingTests.cs" company="Selenium Committers">
+// Licensed to the Software Freedom Conservancy (SFC) under one
+// or more contributor license agreements.  See the NOTICE file
+// distributed with this work for additional information
+// regarding copyright ownership.  The SFC licenses this file
+// to you under the Apache License, Version 2.0 (the
+// "License"); you may not use this file except in compliance
+// with the License.  You may obtain a copy of the License at
+//
+//   http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing,
+// software distributed under the License is distributed on an
+// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+// KIND, either express or implied.  See the License for the
+// specific language governing permissions and limitations
+// under the License.
+// </copyright>
+
 using NUnit.Framework;
 using OpenQA.Selenium.Environment;
 using System;
@@ -13,7 +32,7 @@ namespace OpenQA.Selenium
             driver.Url = formsPage;
             driver.FindElement(By.Id("submitButton")).Click();
             WaitFor(TitleToBe("We Arrive Here"), "Browser title is not 'We Arrive Here'");
-            Assert.AreEqual(driver.Title, "We Arrive Here");
+            Assert.That(driver.Title, Is.EqualTo("We Arrive Here"));
         }
 
         [Test]
@@ -29,7 +48,7 @@ namespace OpenQA.Selenium
             driver.Url = formsPage;
             driver.FindElement(By.Id("imageButton")).Click();
             WaitFor(TitleToBe("We Arrive Here"), "Browser title is not 'We Arrive Here'");
-            Assert.AreEqual(driver.Title, "We Arrive Here");
+            Assert.That(driver.Title, Is.EqualTo("We Arrive Here"));
         }
 
         [Test]
@@ -38,7 +57,7 @@ namespace OpenQA.Selenium
             driver.Url = formsPage;
             driver.FindElement(By.Name("login")).Submit();
             WaitFor(TitleToBe("We Arrive Here"), "Browser title is not 'We Arrive Here'");
-            Assert.AreEqual(driver.Title, "We Arrive Here");
+            Assert.That(driver.Title, Is.EqualTo("We Arrive Here"));
         }
 
         [Test]
@@ -47,7 +66,7 @@ namespace OpenQA.Selenium
             driver.Url = formsPage;
             driver.FindElement(By.Id("checky")).Submit();
             WaitFor(TitleToBe("We Arrive Here"), "Browser title is not 'We Arrive Here'");
-            Assert.AreEqual(driver.Title, "We Arrive Here");
+            Assert.That(driver.Title, Is.EqualTo("We Arrive Here"));
         }
 
         [Test]
@@ -56,7 +75,7 @@ namespace OpenQA.Selenium
             driver.Url = formsPage;
             driver.FindElement(By.XPath("//form/p")).Submit();
             WaitFor(TitleToBe("We Arrive Here"), "Browser title is not 'We Arrive Here'");
-            Assert.AreEqual(driver.Title, "We Arrive Here");
+            Assert.That(driver.Title, Is.EqualTo("We Arrive Here"));
         }
 
         [Test]
@@ -65,7 +84,7 @@ namespace OpenQA.Selenium
             driver.Url = formsPage;
             driver.FindElement(By.Id("submit")).Submit();
             WaitFor(TitleToBe("We Arrive Here"), "Browser title is not 'We Arrive Here'");
-            Assert.AreEqual(driver.Title, "We Arrive Here");
+            Assert.That(driver.Title, Is.EqualTo("We Arrive Here"));
         }
 
         [Test]
@@ -75,7 +94,7 @@ namespace OpenQA.Selenium
             driver.Url = formsPage;
             driver.FindElement(By.Name("submit")).Submit();
             WaitFor(TitleToBe("We Arrive Here"), "Browser title is not 'We Arrive Here'");
-            Assert.AreEqual(driver.Title, "We Arrive Here");
+            Assert.That(driver.Title, Is.EqualTo("We Arrive Here"));
         }
 
         [Test]
@@ -92,7 +111,7 @@ namespace OpenQA.Selenium
             IWebElement textarea = driver.FindElement(By.Id("keyUpArea"));
             string cheesey = "Brie and cheddar";
             textarea.SendKeys(cheesey);
-            Assert.AreEqual(textarea.GetAttribute("value"), cheesey);
+            Assert.That(textarea.GetAttribute("value"), Is.EqualTo(cheesey));
         }
 
         [Test]
@@ -102,7 +121,7 @@ namespace OpenQA.Selenium
             IWebElement textarea = driver.FindElement(By.Id("keyUpArea"));
             string cheesey = "BrIe And CheDdar";
             textarea.SendKeys(cheesey);
-            Assert.AreEqual(textarea.GetAttribute("value"), cheesey);
+            Assert.That(textarea.GetAttribute("value"), Is.EqualTo(cheesey));
         }
 
         [Test]
@@ -115,7 +134,7 @@ namespace OpenQA.Selenium
             input.SendKeys("\n");
 
             WaitFor(TitleToBe("We Arrive Here"), "Browser title is not 'We Arrive Here'");
-            Assert.AreEqual("We Arrive Here", driver.Title);
+            Assert.That(driver.Title, Is.EqualTo("We Arrive Here"));
             Assert.That(driver.Url, Does.EndWith("?x=name"));
         }
 
@@ -128,7 +147,7 @@ namespace OpenQA.Selenium
             input.SendKeys(Keys.Enter);
 
             WaitFor(TitleToBe("We Arrive Here"), "Browser title is not 'We Arrive Here'");
-            Assert.AreEqual("We Arrive Here", driver.Title);
+            Assert.That(driver.Title, Is.EqualTo("We Arrive Here"));
             Assert.That(driver.Url, Does.EndWith("?x=name"));
         }
 
@@ -138,14 +157,14 @@ namespace OpenQA.Selenium
             driver.Url = xhtmlTestPage;
             IWebElement element = driver.FindElement(By.XPath("//form[@name='someForm']/input[@id='username']"));
             String originalValue = element.GetAttribute("value");
-            Assert.AreEqual(originalValue, "change");
+            Assert.That(originalValue, Is.EqualTo("change"));
 
             element.Clear();
             element.SendKeys("some text");
 
             element = driver.FindElement(By.XPath("//form[@name='someForm']/input[@id='username']"));
             String newFormValue = element.GetAttribute("value");
-            Assert.AreEqual(newFormValue, "some text");
+            Assert.That(newFormValue, Is.EqualTo("some text"));
         }
 
         [Test]
@@ -154,7 +173,7 @@ namespace OpenQA.Selenium
             string testFileName = string.Format("test-{0}.txt", Guid.NewGuid().ToString("D"));
             driver.Url = formsPage;
             IWebElement uploadElement = driver.FindElement(By.Id("upload"));
-            Assert.That(uploadElement.GetAttribute("value"), Is.Null.Or.EqualTo(string.Empty));
+            Assert.That(uploadElement.GetAttribute("value"), Is.Null.Or.Empty);
 
             string filePath = System.IO.Path.Combine(EnvironmentManager.Instance.CurrentDirectory, testFileName);
             System.IO.FileInfo inputFile = new System.IO.FileInfo(filePath);
@@ -182,7 +201,7 @@ namespace OpenQA.Selenium
 
             driver.Url = xhtmlFormPage;
             IWebElement uploadElement = driver.FindElement(By.Id("file"));
-            Assert.AreEqual(string.Empty, uploadElement.GetAttribute("value"));
+            Assert.That(uploadElement.GetAttribute("value"), Is.Empty);
 
             string testFileName = string.Format("test-{0}.txt", Guid.NewGuid().ToString("D"));
             string filePath = System.IO.Path.Combine(EnvironmentManager.Instance.CurrentDirectory, testFileName);
@@ -195,7 +214,7 @@ namespace OpenQA.Selenium
 
             string uploadElementValue = uploadElement.GetAttribute("value");
             System.IO.FileInfo outputFile = new System.IO.FileInfo(uploadElementValue.Replace('\\', System.IO.Path.DirectorySeparatorChar));
-            Assert.AreEqual(inputFile.Name, outputFile.Name);
+            Assert.That(outputFile.Name, Is.EqualTo(inputFile.Name));
             inputFile.Delete();
         }
 
@@ -234,11 +253,11 @@ namespace OpenQA.Selenium
             IWebElement element = driver.FindElement(By.Id("working"));
             element.SendKeys("Some");
             String value = element.GetAttribute("value");
-            Assert.AreEqual(value, "Some");
+            Assert.That(value, Is.EqualTo("Some"));
 
             element.SendKeys(" text");
             value = element.GetAttribute("value");
-            Assert.AreEqual(value, "Some text");
+            Assert.That(value, Is.EqualTo("Some text"));
         }
 
         [Test]
@@ -249,7 +268,7 @@ namespace OpenQA.Selenium
             element.SendKeys(". Some text");
             string value = element.GetAttribute("value");
 
-            Assert.AreEqual("Example text. Some text", value);
+            Assert.That(value, Is.EqualTo("Example text. Some text"));
         }
 
         [Test]
@@ -261,7 +280,7 @@ namespace OpenQA.Selenium
             element.SendKeys(". Some text");
             String value = element.GetAttribute("value");
 
-            Assert.AreEqual(value, "Example text. Some text");
+            Assert.That(value, Is.EqualTo("Example text. Some text"));
         }
 
         [Test]
@@ -269,10 +288,10 @@ namespace OpenQA.Selenium
         {
             driver.Url = formsPage;
             IWebElement emptyTextBox = driver.FindElement(By.Id("working"));
-            Assert.AreEqual(emptyTextBox.GetAttribute("value"), "");
+            Assert.That(emptyTextBox.GetAttribute("value"), Is.Empty);
 
             IWebElement emptyTextArea = driver.FindElement(By.Id("emptyTextArea"));
-            Assert.AreEqual(emptyTextBox.GetAttribute("value"), "");
+            Assert.That(emptyTextBox.GetAttribute("value"), Is.Empty);
         }
 
         [Test]
@@ -297,7 +316,7 @@ namespace OpenQA.Selenium
             string text = alert.Text;
             alert.Dismiss();
 
-            Assert.AreEqual("Tasty cheese", text);
+            Assert.That(text, Is.EqualTo("Tasty cheese"));
         }
 
         [Test]
@@ -372,12 +391,12 @@ namespace OpenQA.Selenium
             IWebElement element = driver.FindElement(By.Id("working"));
             element.SendKeys("Some text");
             String value = element.GetAttribute("value");
-            Assert.That(value.Length, Is.GreaterThan(0));
+            Assert.That(value, Is.Not.Empty);
 
             element.Clear();
             value = element.GetAttribute("value");
 
-            Assert.That(value.Length, Is.EqualTo(0));
+            Assert.That(value, Is.Empty);
         }
 
         [Test]
@@ -387,12 +406,12 @@ namespace OpenQA.Selenium
             IWebElement element = driver.FindElement(By.Id("withText"));
             element.SendKeys("Some text");
             String value = element.GetAttribute("value");
-            Assert.That(value.Length, Is.GreaterThan(0));
+            Assert.That(value, Is.Not.Empty);
 
             element.Clear();
             value = element.GetAttribute("value");
 
-            Assert.That(value.Length, Is.EqualTo(0));
+            Assert.That(value, Is.Empty);
         }
 
         private void CheckSubmitButton(string buttonId)
@@ -405,7 +424,7 @@ namespace OpenQA.Selenium
 
             WaitFor(TitleToBe("Submitted Successfully!"), "Browser title is not 'Submitted Successfully!'");
 
-            Assert.That(driver.Url.Contains("name=" + name), "URL does not contain 'name=" + name + "'. Actual URL:" + driver.Url);
+            Assert.That(driver.Url, Does.Contain("name=" + name), "URL does not contain 'name=" + name + "'. Actual URL:" + driver.Url);
         }
 
         private Func<bool> TitleToBe(string desiredTitle)
