@@ -1,3 +1,22 @@
+// <copyright file="VirtualAuthenticatorTest.cs" company="Selenium Committers">
+// Licensed to the Software Freedom Conservancy (SFC) under one
+// or more contributor license agreements.  See the NOTICE file
+// distributed with this work for additional information
+// regarding copyright ownership.  The SFC licenses this file
+// to you under the Apache License, Version 2.0 (the
+// "License"); you may not use this file except in compliance
+// with the License.  You may obtain a copy of the License at
+//
+//   http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing,
+// software distributed under the License is distributed on an
+// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+// KIND, either express or implied.  See the License for the
+// specific language governing permissions and limitations
+// under the License.
+// </copyright>
+
 using NUnit.Framework;
 using OpenQA.Selenium.Environment;
 using OpenQA.Selenium.Internal;
@@ -52,7 +71,8 @@ namespace OpenQA.Selenium.VirtualAuth
         [TearDown]
         public void Teardown()
         {
-            if (webDriver.AuthenticatorId != null)
+            if (webDriver.AuthenticatorId is not null &&
+                webDriver.SessionId is not null)
             {
                 webDriver.RemoveVirtualAuthenticator(webDriver.AuthenticatorId);
             }
@@ -150,12 +170,12 @@ namespace OpenQA.Selenium.VirtualAuth
             object response = jsDriver.ExecuteAsyncScript(
               "registerCredential().then(arguments[arguments.length - 1]);");
 
-            Assert.AreEqual("OK", ((Dictionary<string, object>)response)["status"]);
+            Assert.That((Dictionary<string, object>)response, Does.ContainKey("status").WithValue("OK"));
 
             object assertionResponse = GetAssertionFor(ExtractRawIdFrom(response));
 
             // Attempt to use the credential to get an assertion.
-            Assert.AreEqual("OK", ((Dictionary<string, object>)assertionResponse)["status"]);
+            Assert.That((Dictionary<string, object>)assertionResponse, Does.ContainKey("status").WithValue("OK"));
         }
 
         [Test]
@@ -167,9 +187,31 @@ namespace OpenQA.Selenium.VirtualAuth
         {
             VirtualAuthenticatorOptions options = new VirtualAuthenticatorOptions();
             string authenticatorId = webDriver.AddVirtualAuthenticator(options);
+            Assert.That(webDriver.AuthenticatorId, Is.EqualTo(authenticatorId));
+
             webDriver.RemoveVirtualAuthenticator(authenticatorId);
 
-            Assert.IsNull(webDriver.AuthenticatorId);
+            Assert.That(webDriver.AuthenticatorId, Is.Null);
+        }
+
+        [Test]
+        [NeedsFreshDriver(IsCreatedAfterTest = true)]
+        [IgnoreBrowser(Selenium.Browser.IE, "IE does not support Virtual Authenticator")]
+        [IgnoreBrowser(Selenium.Browser.Firefox, "Firefox does not support Virtual Authenticator")]
+        [IgnoreBrowser(Selenium.Browser.Safari, "Safari does not support Virtual Authenticator")]
+        public void ShouldSupportMultipleVirtualAuthenticatorsAtOnce()
+        {
+            VirtualAuthenticatorOptions options = new VirtualAuthenticatorOptions();
+
+            string authenticatorId1 = webDriver.AddVirtualAuthenticator(options);
+            Assert.That(webDriver.AuthenticatorId, Is.EqualTo(authenticatorId1));
+
+            string authenticatorId2 = webDriver.AddVirtualAuthenticator(options);
+
+            webDriver.RemoveVirtualAuthenticator(authenticatorId1);
+            webDriver.RemoveVirtualAuthenticator(authenticatorId2);
+
+            Assert.That(webDriver.AuthenticatorId, Is.Null);
         }
 
         [Test]
@@ -194,7 +236,7 @@ namespace OpenQA.Selenium.VirtualAuth
 
             // Attempt to use the credential to generate an assertion.
             object response = GetAssertionFor(id);
-            Assert.AreEqual("OK", ((Dictionary<string, object>)response)["status"]);
+            Assert.That((Dictionary<string, object>)response, Does.ContainKey("status").WithValue("OK"));
         }
 
         [Test]
@@ -226,7 +268,7 @@ namespace OpenQA.Selenium.VirtualAuth
 
             // Attempt to use the credential to generate an assertion.
             object response = GetAssertionFor(id);
-            Assert.AreEqual("OK", ((Dictionary<string, object>)response)["status"]);
+            Assert.That((Dictionary<string, object>)response, Does.ContainKey("status").WithValue("OK"));
         }
 
         [Test]
@@ -250,14 +292,14 @@ namespace OpenQA.Selenium.VirtualAuth
             // empty allowCredentials array.
             object response = jsDriver.ExecuteAsyncScript(
               "getCredential([]).then(arguments[arguments.length - 1]);");
-            Assert.AreEqual("OK", ((Dictionary<string, object>)response)["status"]);
+            Assert.That(((Dictionary<string, object>)response)["status"], Is.EqualTo("OK"));
 
             Dictionary<string, object> attestation = (Dictionary<string, object>)((Dictionary<string, object>)response)["attestation"];
 
             ReadOnlyCollection<object> returnedUserHandle = (ReadOnlyCollection<object>)attestation["userHandle"];
 
-            Assert.AreEqual(1, returnedUserHandle.Count);
-            Assert.AreEqual(0, returnedUserHandle.IndexOf(1L));
+            Assert.That(returnedUserHandle, Has.One.Items);
+            Assert.That(returnedUserHandle.IndexOf(1L), Is.Zero);
         }
 
         [Test]
@@ -280,7 +322,9 @@ namespace OpenQA.Selenium.VirtualAuth
             byte[] userHandle = { 1 };
             Credential credential = Credential.CreateResidentCredential(
               credentialId, "localhost", base64EncodedEC256PK, userHandle, /*signCount=*/0);
-            Assert.Throws<WebDriverArgumentException>(() => webDriver.AddCredential(credential));
+            Assert.That(
+                () => webDriver.AddCredential(credential),
+                Throws.TypeOf<WebDriverArgumentException>().With.Message.Contains("The Authenticator does not support Resident Credentials."));
         }
 
         [Test]
@@ -297,21 +341,21 @@ namespace OpenQA.Selenium.VirtualAuth
             object response1 = jsDriver.ExecuteAsyncScript(
               "registerCredential({authenticatorSelection: {requireResidentKey: true}})"
               + " .then(arguments[arguments.length - 1]);");
-            Assert.AreEqual("OK", ((Dictionary<string, object>)response1)["status"]);
+            Assert.That((Dictionary<string, object>)response1, Does.ContainKey("status").WithValue("OK"));
 
             // Register a non resident credential.
             object response2 = jsDriver.ExecuteAsyncScript(
               "registerCredential().then(arguments[arguments.length - 1]);");
-            Assert.AreEqual("OK", ((Dictionary<string, object>)response2)["status"]);
+            Assert.That((Dictionary<string, object>)response2, Does.ContainKey("status").WithValue("OK"));
 
             byte[] credential1Id = ConvertListIntoArrayOfBytes(ExtractRawIdFrom(response1));
             byte[] credential2Id = ConvertListIntoArrayOfBytes(ExtractRawIdFrom(response2));
 
-            Assert.AreNotEqual(credential1Id, credential2Id);
+            Assert.That(credential2Id, Is.Not.EqualTo(credential1Id));
 
             // Retrieve the two credentials.
             List<Credential> credentials = webDriver.GetCredentials();
-            Assert.AreEqual(2, credentials.Count);
+            Assert.That(credentials, Has.Exactly(2).Items);
 
             Credential credential1 = null;
             Credential credential2 = null;
@@ -331,11 +375,11 @@ namespace OpenQA.Selenium.VirtualAuth
                 }
             }
 
-            Assert.True(credential1.IsResidentCredential);
-            Assert.NotNull(credential1.PrivateKey);
+            Assert.That(credential1.IsResidentCredential, Is.True);
+            Assert.That(credential1.PrivateKey, Is.Not.Null);
 
-            Assert.False(credential2.IsResidentCredential);
-            Assert.NotNull(credential2.PrivateKey);
+            Assert.That(credential2.IsResidentCredential, Is.False);
+            Assert.That(credential2.PrivateKey, Is.Not.Null);
         }
 
         [Test]
@@ -351,7 +395,7 @@ namespace OpenQA.Selenium.VirtualAuth
             object response = jsDriver.ExecuteAsyncScript(
               "registerCredential().then(arguments[arguments.length - 1]);");
 
-            Assert.AreEqual("OK", ((Dictionary<string, object>)response)["status"]);
+            Assert.That((Dictionary<string, object>)response, Does.ContainKey("status").WithValue("OK"));
 
             // Remove a credential by its ID as an array of bytes.
             List<long> rawId = ExtractRawIdFrom(response);
@@ -362,7 +406,7 @@ namespace OpenQA.Selenium.VirtualAuth
             object assertionResponse = GetAssertionFor(rawId);
             string error = (string)((Dictionary<string, object>)assertionResponse)["status"];
 
-            Assert.True(error.StartsWith("NotAllowedError"));
+            Assert.That(error, Does.StartWith("NotAllowedError"));
         }
 
         [Test]
@@ -378,7 +422,7 @@ namespace OpenQA.Selenium.VirtualAuth
             object response = jsDriver.ExecuteAsyncScript(
               "registerCredential().then(arguments[arguments.length - 1]);");
 
-            Assert.AreEqual("OK", ((Dictionary<string, object>)response)["status"]);
+            Assert.That((Dictionary<string, object>)response, Does.ContainKey("status").WithValue("OK"));
 
             // Remove a credential by its base64url ID.
             String credentialId = ExtractIdFrom(response);
@@ -388,7 +432,7 @@ namespace OpenQA.Selenium.VirtualAuth
             object assertionResponse = GetAssertionFor(credentialId);
             string error = (string)((Dictionary<string, object>)assertionResponse)["status"];
 
-            Assert.True(error.StartsWith("NotAllowedError"));
+            Assert.That(error, Does.StartWith("NotAllowedError"));
         }
 
         [Test]
@@ -403,12 +447,12 @@ namespace OpenQA.Selenium.VirtualAuth
             // Register two credentials.
             object response1 = jsDriver.ExecuteAsyncScript(
               "registerCredential().then(arguments[arguments.length - 1]);");
-            Assert.AreEqual("OK", ((Dictionary<string, object>)response1)["status"]);
+            Assert.That((Dictionary<string, object>)response1, Does.ContainKey("status").WithValue("OK"));
             List<long> rawId1 = ExtractRawIdFrom(response1);
 
             object response2 = jsDriver.ExecuteAsyncScript(
               "registerCredential().then(arguments[arguments.length - 1]);");
-            Assert.AreEqual("OK", ((Dictionary<string, object>)response2)["status"]);
+            Assert.That((Dictionary<string, object>)response2, Does.ContainKey("status").WithValue("OK"));
             List<long> rawId2 = ExtractRawIdFrom(response1);
 
             // Remove all credentials.
@@ -427,7 +471,7 @@ namespace OpenQA.Selenium.VirtualAuth
 
             string error = (string)((Dictionary<string, object>)response)["status"];
 
-            Assert.True(error.StartsWith("NotAllowedError"));
+            Assert.That(error, Does.StartWith("NotAllowedError"));
         }
 
         [Test]
@@ -435,7 +479,7 @@ namespace OpenQA.Selenium.VirtualAuth
         [IgnoreBrowser(Selenium.Browser.IE, "IE does not support Virtual Authenticator")]
         [IgnoreBrowser(Selenium.Browser.Firefox, "Firefox does not support Virtual Authenticator")]
         [IgnoreBrowser(Selenium.Browser.Safari, "Safari does not support Virtual Authenticator")]
-        public void testSetUserVerified()
+        public void TestSetUserVerified()
         {
             CreateRKEnabledCTAP2Authenticator();
 
@@ -443,7 +487,7 @@ namespace OpenQA.Selenium.VirtualAuth
             Object response = jsDriver.ExecuteAsyncScript(
               "registerCredential({authenticatorSelection: {userVerification: 'required'}})"
               + "  .then(arguments[arguments.length - 1]);");
-            Assert.AreEqual("OK", ((Dictionary<string, object>)response)["status"]);
+            Assert.That((Dictionary<string, object>)response, Does.ContainKey("status").WithValue("OK"));
             List<long> rawId = ExtractRawIdFrom(response);
 
             // Getting an assertion requiring user verification should succeed.
@@ -453,7 +497,7 @@ namespace OpenQA.Selenium.VirtualAuth
               + "  \"id\": Int8Array.from(arguments[0]),"
               + "}], {userVerification: 'required'}).then(arguments[arguments.length - 1]);",
               rawId);
-            Assert.AreEqual("OK", ((Dictionary<string, object>)response)["status"]);
+            Assert.That((Dictionary<string, object>)response, Does.ContainKey("status").WithValue("OK"));
 
             // Disable user verification.
             webDriver.SetUserVerified(false);
@@ -468,7 +512,39 @@ namespace OpenQA.Selenium.VirtualAuth
 
             string error = (string)((Dictionary<string, object>)response)["status"];
 
-            Assert.True(error.StartsWith("NotAllowedError"));
+            Assert.That(error, Does.StartWith("NotAllowedError"));
+        }
+
+        [Test]
+        [NeedsFreshDriver(IsCreatedAfterTest = true)]
+        [IgnoreBrowser(Selenium.Browser.IE, "IE does not support Virtual Authenticator")]
+        [IgnoreBrowser(Selenium.Browser.Firefox, "Firefox does not support Virtual Authenticator")]
+        [IgnoreBrowser(Selenium.Browser.Safari, "Safari does not support Virtual Authenticator")]
+        public void ShouldThrowOnInvalidArguments()
+        {
+            Assert.That(
+                () => webDriver.AddVirtualAuthenticator(null),
+                Throws.ArgumentNullException);
+
+            Assert.That(
+                () => webDriver.RemoveVirtualAuthenticator(null),
+                Throws.ArgumentNullException);
+
+            Assert.That(
+                () => webDriver.AddCredential(null),
+                Throws.ArgumentNullException);
+
+            Assert.That(
+               () => webDriver.RemoveCredential((byte[])null),
+               Throws.ArgumentNullException);
+
+            Assert.That(
+                () => webDriver.RemoveCredential((string)null),
+                Throws.ArgumentNullException);
+
+            Assert.That(
+               () => webDriver.RemoveVirtualAuthenticator("non-existant"),
+               Throws.TypeOf<WebDriverArgumentException>());
         }
     }
 }
