@@ -15,13 +15,19 @@
 # specific language governing permissions and limitations
 # under the License.
 
+import os
 from unittest.mock import patch
 from urllib import parse
 
 import pytest
 import urllib3
+from urllib3.util import Retry
+from urllib3.util import Timeout
 
 from selenium import __version__
+from selenium.webdriver import Proxy
+from selenium.webdriver.common.proxy import ProxyType
+from selenium.webdriver.remote.client_config import AuthType
 from selenium.webdriver.remote.remote_connection import ClientConfig
 from selenium.webdriver.remote.remote_connection import RemoteConnection
 
@@ -64,8 +70,13 @@ def test_get_remote_connection_headers_defaults():
 
 def test_get_remote_connection_headers_adds_auth_header_if_pass():
     url = "http://user:pass@remote"
-    headers = RemoteConnection.get_remote_connection_headers(parse.urlparse(url))
+    with pytest.warns(None) as record:
+        headers = RemoteConnection.get_remote_connection_headers(parse.urlparse(url))
     assert headers.get("Authorization") == "Basic dXNlcjpwYXNz"
+    assert (
+        record[0].message.args[0]
+        == "Embedding username and password in URL could be insecure, use ClientConfig instead"
+    )
 
 
 def test_get_remote_connection_headers_adds_keep_alive_if_requested():
@@ -81,15 +92,33 @@ def test_get_proxy_url_http(mock_proxy_settings):
     assert proxy_url == proxy
 
 
-def test_get_auth_header_if_client_config_pass():
+def test_get_auth_header_if_client_config_pass_basic_auth():
     custom_config = ClientConfig(
-        remote_server_addr="http://remote", keep_alive=True, username="user", password="pass", auth_type="Basic"
+        remote_server_addr="http://remote", keep_alive=True, username="user", password="pass", auth_type=AuthType.BASIC
     )
     remote_connection = RemoteConnection(custom_config.remote_server_addr, client_config=custom_config)
     headers = remote_connection._client_config.get_auth_header()
     assert headers.get("Authorization") == "Basic dXNlcjpwYXNz"
 
 
+def test_get_auth_header_if_client_config_pass_bearer_token():
+    custom_config = ClientConfig(
+        remote_server_addr="http://remote", keep_alive=True, auth_type=AuthType.BEARER, token="dXNlcjpwYXNz"
+    )
+    remote_connection = RemoteConnection(custom_config.remote_server_addr, client_config=custom_config)
+    headers = remote_connection._client_config.get_auth_header()
+    assert headers.get("Authorization") == "Bearer dXNlcjpwYXNz"
+
+
+def test_get_auth_header_if_client_config_pass_x_api_key():
+    custom_config = ClientConfig(
+        remote_server_addr="http://remote", keep_alive=True, auth_type=AuthType.X_API_KEY, token="abcdefgh123456789"
+    )
+    remote_connection = RemoteConnection(custom_config.remote_server_addr, client_config=custom_config)
+    headers = remote_connection._client_config.get_auth_header()
+    assert headers.get("X-API-Key") == "abcdefgh123456789"
+
+
 def test_get_proxy_url_https(mock_proxy_settings):
     proxy = "http://https_proxy.com:8080"
     remote_connection = RemoteConnection("https://remote", keep_alive=False)
@@ -97,6 +126,62 @@ def test_get_proxy_url_https(mock_proxy_settings):
     assert proxy_url == proxy
 
 
+def test_get_proxy_url_https_via_client_config():
+    client_config = ClientConfig(
+        remote_server_addr="https://localhost:4444",
+        proxy=Proxy({"proxyType": ProxyType.MANUAL, "sslProxy": "https://admin:admin@http_proxy.com:8080"}),
+    )
+    remote_connection = RemoteConnection(client_config=client_config)
+    conn = remote_connection._get_connection_manager()
+    assert isinstance(conn, urllib3.ProxyManager)
+    conn.proxy_url = "https://http_proxy.com:8080"
+    conn.connection_pool_kw["proxy_headers"] = urllib3.make_headers(proxy_basic_auth="admin:admin")
+
+
+def test_get_proxy_url_http_via_client_config():
+    client_config = ClientConfig(
+        remote_server_addr="http://localhost:4444",
+        proxy=Proxy(
+            {
+                "proxyType": ProxyType.MANUAL,
+                "httpProxy": "http://admin:admin@http_proxy.com:8080",
+                "sslProxy": "https://admin:admin@http_proxy.com:8080",
+            }
+        ),
+    )
+    remote_connection = RemoteConnection(client_config=client_config)
+    conn = remote_connection._get_connection_manager()
+    assert isinstance(conn, urllib3.ProxyManager)
+    conn.proxy_url = "http://http_proxy.com:8080"
+    conn.connection_pool_kw["proxy_headers"] = urllib3.make_headers(proxy_basic_auth="admin:admin")
+
+
+def test_get_proxy_direct_via_client_config():
+    client_config = ClientConfig(
+        remote_server_addr="http://localhost:4444", proxy=Proxy({"proxyType": ProxyType.DIRECT})
+    )
+    remote_connection = RemoteConnection(client_config=client_config)
+    conn = remote_connection._get_connection_manager()
+    assert isinstance(conn, urllib3.PoolManager)
+    proxy_url = remote_connection._client_config.get_proxy_url()
+    assert proxy_url is None
+
+
+def test_get_proxy_system_matches_no_proxy_via_client_config():
+    os.environ["HTTP_PROXY"] = "http://admin:admin@system_proxy.com:8080"
+    os.environ["NO_PROXY"] = "localhost,127.0.0.1"
+    client_config = ClientConfig(
+        remote_server_addr="http://localhost:4444", proxy=Proxy({"proxyType": ProxyType.SYSTEM})
+    )
+    remote_connection = RemoteConnection(client_config=client_config)
+    conn = remote_connection._get_connection_manager()
+    assert isinstance(conn, urllib3.PoolManager)
+    proxy_url = remote_connection._client_config.get_proxy_url()
+    assert proxy_url is None
+    os.environ.pop("HTTP_PROXY")
+    os.environ.pop("NO_PROXY")
+
+
 def test_get_proxy_url_none(mock_proxy_settings_missing):
     remote_connection = RemoteConnection("https://remote", keep_alive=False)
     proxy_url = remote_connection._client_config.get_proxy_url()
@@ -295,6 +380,28 @@ def test_override_user_agent_in_headers(mock_get_remote_connection_headers, remo
     assert headers.get("Content-Type") == "application/json;charset=UTF-8"
 
 
+@patch("selenium.webdriver.remote.remote_connection.RemoteConnection.get_remote_connection_headers")
+def test_override_user_agent_via_client_config(mock_get_remote_connection_headers):
+    client_config = ClientConfig(
+        remote_server_addr="http://localhost:4444",
+        user_agent="custom-agent/1.0 (python 3.8)",
+        extra_headers={"Content-Type": "application/xml;charset=UTF-8"},
+    )
+    remote_connection = RemoteConnection(client_config=client_config)
+
+    mock_get_remote_connection_headers.return_value = {
+        "Accept": "application/json",
+        "Content-Type": "application/xml;charset=UTF-8",
+        "User-Agent": "custom-agent/1.0 (python 3.8)",
+    }
+
+    headers = remote_connection.get_remote_connection_headers(parse.urlparse("http://localhost:4444"))
+
+    assert headers.get("User-Agent") == "custom-agent/1.0 (python 3.8)"
+    assert headers.get("Accept") == "application/json"
+    assert headers.get("Content-Type") == "application/xml;charset=UTF-8"
+
+
 @patch("selenium.webdriver.remote.remote_connection.RemoteConnection._request")
 def test_register_extra_headers(mock_request, remote_connection):
     RemoteConnection.extra_headers = {"Foo": "bar"}
@@ -307,6 +414,39 @@ def test_register_extra_headers(mock_request, remote_connection):
     assert headers["Foo"] == "bar"
 
 
+@patch("selenium.webdriver.remote.remote_connection.RemoteConnection._request")
+def test_register_extra_headers_via_client_config(mock_request):
+    client_config = ClientConfig(
+        remote_server_addr="http://localhost:4444",
+        extra_headers={
+            "Authorization": "AWS4-HMAC-SHA256",
+            "Credential": "abc/20200618/us-east-1/execute-api/aws4_request",
+        },
+    )
+    remote_connection = RemoteConnection(client_config=client_config)
+
+    mock_request.return_value = {"status": 200, "value": "OK"}
+    remote_connection.execute("newSession", {})
+
+    mock_request.assert_called_once_with("POST", "http://localhost:4444/session", body="{}")
+    headers = remote_connection.get_remote_connection_headers(parse.urlparse("http://localhost:4444"), False)
+    assert headers["Authorization"] == "AWS4-HMAC-SHA256"
+    assert headers["Credential"] == "abc/20200618/us-east-1/execute-api/aws4_request"
+
+
+def test_backwards_compatibility_with_appium_connection():
+    # Keep backward compatibility for AppiumConnection - https://github.com/SeleniumHQ/selenium/issues/14694
+    client_config = ClientConfig(remote_server_addr="http://remote", ca_certs="/path/to/cacert.pem", timeout=300)
+    remote_connection = RemoteConnection(client_config=client_config)
+    assert remote_connection._ca_certs == "/path/to/cacert.pem"
+    assert remote_connection._timeout == 300
+    assert remote_connection._client_config == client_config
+    remote_connection.set_timeout(120)
+    assert remote_connection.get_timeout() == 120
+    remote_connection.set_certificate_bundle_path("/path/to/cacert2.pem")
+    assert remote_connection.get_certificate_bundle_path() == "/path/to/cacert2.pem"
+
+
 def test_get_connection_manager_with_timeout_from_client_config():
     remote_connection = RemoteConnection(remote_server_addr="http://remote", keep_alive=False)
     remote_connection.set_timeout(10)
@@ -315,6 +455,8 @@ def test_get_connection_manager_with_timeout_from_client_config():
     assert conn.connection_pool_kw["timeout"] == 10
     assert isinstance(conn, urllib3.PoolManager)
 
+
+def test_connection_manager_with_timeout_via_client_config():
     client_config = ClientConfig("http://remote", timeout=300)
     remote_connection = RemoteConnection(client_config=client_config)
     conn = remote_connection._get_connection_manager()
@@ -322,7 +464,7 @@ def test_get_connection_manager_with_timeout_from_client_config():
     assert isinstance(conn, urllib3.PoolManager)
 
 
-def test_get_connection_manager_with_ca_certs_from_client_config():
+def test_get_connection_manager_with_ca_certs():
     remote_connection = RemoteConnection(remote_server_addr="http://remote")
     remote_connection.set_certificate_bundle_path("/path/to/cacert.pem")
     conn = remote_connection._get_connection_manager()
@@ -331,6 +473,8 @@ def test_get_connection_manager_with_ca_certs_from_client_config():
     assert conn.connection_pool_kw["ca_certs"] == "/path/to/cacert.pem"
     assert isinstance(conn, urllib3.PoolManager)
 
+
+def test_connection_manager_with_ca_certs_via_client_config():
     client_config = ClientConfig(remote_server_addr="http://remote", ca_certs="/path/to/cacert.pem")
     remote_connection = RemoteConnection(client_config=client_config)
     conn = remote_connection._get_connection_manager()
@@ -348,15 +492,17 @@ def test_get_connection_manager_ignores_certificates():
     assert conn.connection_pool_kw["cert_reqs"] == "CERT_NONE"
     assert isinstance(conn, urllib3.PoolManager)
 
+    remote_connection.reset_timeout()
+    assert remote_connection.get_timeout() is None
+
+
+def test_connection_manager_ignores_certificates_via_client_config():
     client_config = ClientConfig(remote_server_addr="http://remote", ignore_certificates=True, timeout=10)
     remote_connection = RemoteConnection(client_config=client_config)
     conn = remote_connection._get_connection_manager()
+    assert isinstance(conn, urllib3.PoolManager)
     assert conn.connection_pool_kw["timeout"] == 10
     assert conn.connection_pool_kw["cert_reqs"] == "CERT_NONE"
-    assert isinstance(conn, urllib3.PoolManager)
-
-    remote_connection.reset_timeout()
-    assert remote_connection.get_timeout() is None
 
 
 def test_get_connection_manager_with_custom_args():
@@ -370,11 +516,16 @@ def test_get_connection_manager_with_custom_args():
     assert conn.connection_pool_kw["retries"] == 3
     assert conn.connection_pool_kw["block"] is True
 
+
+def test_connection_manager_with_custom_args_via_client_config():
+    retries = Retry(connect=2, read=2, redirect=2)
+    timeout = Timeout(connect=300, read=3600)
     client_config = ClientConfig(
-        remote_server_addr="http://remote", keep_alive=False, init_args_for_pool_manager=custom_args
+        remote_server_addr="http://localhost:4444",
+        init_args_for_pool_manager={"init_args_for_pool_manager": {"retries": retries, "timeout": timeout}},
     )
     remote_connection = RemoteConnection(client_config=client_config)
     conn = remote_connection._get_connection_manager()
     assert isinstance(conn, urllib3.PoolManager)
-    assert conn.connection_pool_kw["retries"] == 3
-    assert conn.connection_pool_kw["block"] is True
+    assert conn.connection_pool_kw["retries"] == retries
+    assert conn.connection_pool_kw["timeout"] == timeout
