@@ -7,10 +7,9 @@ set -x
 # Now run the tests. The engflow build uses pinned browsers
 # so this should be fine
 # shellcheck disable=SC2046
-bazel test --config=remote-ci --build_tests_only \
-  --test_tag_filters=-exclusive-if-local,-skip-remote \
+bazel test --config=rbe-ci --build_tests_only \
   --keep_going --flaky_test_attempts=2 \
   //... -- $(cat .skipped-tests | tr '\n' ' ')
 
 # Build the packages we want to ship to users
-bazel build --config=remote-ci --build_tag_filters=release-artifact //...
+bazel build --config=rbe-ci --build_tag_filters=release-artifact //...
